from django.db import models
#from reqman.apps.common.models import CoreModel
from reqman.apps.account.models import User
from reqman.apps.reqtool.models.main_req import create_main, ITEM_CHOICES, STATE_VALUE_CHOICES, STATE_CHOICES, SYSTEM_CHOICES, FUNCTION_CHOICES, FLOW_CHOICES
from reqman.apps.reqtool.models.suffix_req import create_suffix
from reqman.apps.reqtool.rest_api.services.parse_ontologies import *

from pygments.formatters.html import HtmlFormatter
from pygments import highlight

#get prefix syntax from the Onotlogy
get_prefix_syntax()

SIMPLE_PREFIX_CHOICES = (   ("",""),  
                            ("P1", "If/Unless <logical expression>"), 
                            ("P2", "As soon as <occuring functionality>"), 
                            ("P3", "As long as <occuring functionality>")
                        )

LOGIC_CONNECTIVE = (    ("or","OR"), 
                        ("and", "AND"), 
                        ("xor", "XOR")
                    )

LOGICAL_EXPRESSION = (  ("",""), 
                        ("state value constraint", "state value constraint"), 
                        ("occuring functionality", "occuring functionality")
                    )

VERB_CHOICES = (("is","is"),("send","send"),("receive","receive"),
                ("ingest","ingest"),("emit","emit"),("perform","perform"))


PREFIX_CHOICES = (   ("If/Unless", "If/Unless"), 
                            ("As soon as", "As soon as"), 
                            ("As long as", "As long as")
                        )


def create_prefixgroup(instance):
    '''Create the PrefixGroup Model for the Boilerplate'''
    try :
        prefix = PrefixGroup.objects.get(boilerplate = instance, prefix_owner = instance.owner)
        if instance.has_prefix:
            return
        prefix = PrefixGroup.objects.get(boilerplate = instance, prefix_owner = instance.owner)
        prefix.delete()
    except PrefixGroup.DoesNotExist:
        if instance.has_prefix:
            prefix = PrefixGroup(boilerplate = instance, prefix_owner = instance.owner)
            prefix.save()


def create_prefix(instance):
        prefix = Prefix(prefix_boilerplate = instance.boilerplate, prefixgroup = instance)
        prefix.save()


def create_extra_prefix(instance, now_prefix):
    prefix = Prefix(prefixgroup = instance.prefixgroup, prefix_boilerplate = instance.prefix_boilerplate)
    prefix.save()


class Boilerplate(models.Model):
    owner = models.ForeignKey(User, related_name='boilerplate', on_delete=models.CASCADE)    
    created = models.DateTimeField(auto_now_add=True)
    has_prefix = models.BooleanField(default=False)
    has_main = models.BooleanField(default=False)
    has_suffix = models.BooleanField(default=False)
     

    def save(self, **kwargs):
        """Create prefix, main or suffix """
        super(Boilerplate, self).save(**kwargs)
        create_prefixgroup(self)
        create_main(self)
        create_suffix(self)
        
        
    class Meta:
        ordering = ['created']


class PrefixGroup(models.Model):
    boilerplate = models.ForeignKey(Boilerplate, related_name='prefixgroup', on_delete=models.CASCADE)
    prefix_owner = models.ForeignKey(User, related_name='prefix_group_owner', on_delete=models.CASCADE)

    def save(self, **kwargs):
        super(PrefixGroup, self).save(**kwargs)
        create_prefix(self)
        


class Prefix(models.Model):
    #boilerplate = models.OneToOneField(Boilerplate, related_name='prefix', on_delete=models.CASCADE, primary_key=True)
    prefixgroup = models.ForeignKey(PrefixGroup, related_name='prefix', on_delete=models.CASCADE)
    prefix_boilerplate = models.ForeignKey(Boilerplate, related_name='prefix_boilerplate', on_delete=models.CASCADE)
    prefix = models.CharField(choices=PREFIX_CHOICES, max_length=100)
    #state value constraint
    system_fun_item = models.CharField(choices= (ITEM_CHOICES + SYSTEM_CHOICES + FUNCTION_CHOICES), max_length=100, blank=True)
    state_or_verb =  models.CharField(choices=STATE_CHOICES + VERB_CHOICES, max_length=100, blank=True)
    #state_value = models.CharField(choices=STATE_VALUE_CHOICES, max_length=100, blank=True)
    #occuring functionality
    #sys_fun = models.CharField(choices=, max_length=100, blank=True)
    #verb = models.CharField(choices=VERB_CHOICES, max_length=100, blank=True)
    item_function_flow_statevalue = models.CharField(choices=ITEM_CHOICES + FLOW_CHOICES + FUNCTION_CHOICES + STATE_VALUE_CHOICES, max_length=100, blank=True)
    #Prefix value P1/P2/P3
    logical_expression =  models.CharField(choices=LOGICAL_EXPRESSION, max_length=100, blank=True)
    #Info for prefix
    logic_connective = models.CharField(choices=LOGIC_CONNECTIVE, default='', max_length=100, blank=True)
    simple_prefix = models.CharField(choices=SIMPLE_PREFIX_CHOICES, max_length=100)


    def __init__(self, *args, **kwargs):
        super(Prefix, self).__init__(*args, **kwargs)
        self.__important_fields = ['logic_connective']
        for field in self.__important_fields:
            setattr(self, '__original_%s' % field, getattr(self, field))


    def save(self, **kwargs):
        """Create P1, P2 or P3 """
        prev_prefix_list = []
        super(Prefix, self).save(**kwargs)
        for field in self.__important_fields:
            orig = '__original_%s' % field
            previous_simple_prefix = getattr(self, orig)
            prev_prefix_list.append(previous_simple_prefix)
        if previous_simple_prefix == "" and not self.logic_connective == "" :
            create_extra_prefix ( self, self.logic_connective)
        

    class Meta:
        ordering = ['prefix_boilerplate']

